__author__ = "Martin Uhrin"
__copyright__ = "Copyright 2022, Martin Uhrin"
__license__ = "LGPL-3.0-or-later"

version_info = (0, 1, 3)
__version__ = ".".join(map(str, version_info))
__maintainer__ = "Martin Uhrin"
__email__ = "martin.uhrin.10@ucl.ac.uk"
__status__ = "Development"

__all__ = "__author__", "__copyright__", "__version__", "__license__"
